package com.sun.jmx.mbeanserver;

import java.util.Comparator;
import java.util.Set;
import java.util.Arrays;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import javax.management.NotCompliantMBeanException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

class MBeanAnalyzer<M>
{
    private Map<String, List<M>> opMap;
    private Map<String, MBeanAnalyzer.AttrMethods<M>> attrMap;
    
    void visit(MBeanAnalyzer.MBeanVisitor<M> mBeanVisitor) {
        for (Map.Entry entry : this.attrMap.entrySet()) {
            String s = (String)entry.getKey();
            MBeanAnalyzer.AttrMethods<Object> attrMethods = (MBeanAnalyzer.AttrMethods)entry.getValue();
            mBeanVisitor.visitAttribute(s, attrMethods.getter, attrMethods.setter);
        }
        for (Map.Entry entry2 : this.opMap.entrySet()) {
            Iterator<Object> iterator3 = ((List)entry2.getValue()).iterator();
            while (iterator3.hasNext()) {
                mBeanVisitor.visitOperation((String)entry2.getKey(), iterator3.next());
            }
        }
    }
    
    static <M> MBeanAnalyzer<M> analyzer(Class<?> clazz, MBeanIntrospector<M> mBeanIntrospector) throws NotCompliantMBeanException {
        return new MBeanAnalyzer(clazz, mBeanIntrospector);
    }
    
    private MBeanAnalyzer(Class<?> clazz, MBeanIntrospector<M> mBeanIntrospector) throws NotCompliantMBeanException {
        this.opMap = Util.newInsertionOrderMap();
        this.attrMap = Util.newInsertionOrderMap();
        if (!clazz.isInterface()) {
            throw new NotCompliantMBeanException("Not an interface: " + clazz.getName());
        }
        if (!Modifier.isPublic(clazz.getModifiers()) && !Introspector.ALLOW_NONPUBLIC_MBEAN) {
            throw new NotCompliantMBeanException("Interface is not public: " + clazz.getName());
        }
        try {
            this.initMaps(clazz, mBeanIntrospector);
        }
        catch (Exception ex) {
            throw Introspector.throwException(clazz, ex);
        }
        return;
    }
    
    private void initMaps(Class<?> clazz, MBeanIntrospector<M> mBeanIntrospector) throws Exception {
        for (Method method : eliminateCovariantMethods(mBeanIntrospector.getMethods(clazz))) {
            String name = method.getName();
            int length = method.getParameterTypes().length;
            Object m = mBeanIntrospector.mFrom(method);
            String str = "";
            if (name.startsWith("get")) {
                str = name.substring(3);
            }
            else if (name.startsWith("is") && method.getReturnType() == Boolean.TYPE) {
                str = name.substring(2);
            }
            if (str.length() != 0 && length == 0 && method.getReturnType() != Void.TYPE) {
                MBeanAnalyzer.AttrMethods<Object> attrMethods2 = (MBeanAnalyzer.AttrMethods)this.attrMap.get(str);
                if (attrMethods2 == null) {
                    attrMethods2 = new MBeanAnalyzer.AttrMethods();
                }
                else if (attrMethods2.getter != null) {
                    throw new NotCompliantMBeanException("Attribute " + str + " has more than one getter");
                }
                attrMethods2.getter = m;
                this.attrMap.put(str, attrMethods2);
            }
            else if (name.startsWith("set") && name.length() > 3 && length == 1 && method.getReturnType() == Void.TYPE) {
                String substring = name.substring(3);
                MBeanAnalyzer.AttrMethods<Object> attrMethods = (MBeanAnalyzer.AttrMethods)this.attrMap.get(substring);
                if (attrMethods == null) {
                    attrMethods = new MBeanAnalyzer.AttrMethods();
                }
                else if (attrMethods.setter != null) {
                    throw new NotCompliantMBeanException("Attribute " + substring + " has more than one setter");
                }
                attrMethods.setter = m;
                this.attrMap.put(substring, attrMethods);
            }
            else {
                List<Object> list = (List)this.opMap.get(name);
                if (list == null) {
                    list = Util.newList();
                }
                list.add(m);
                this.opMap.put(name, list);
            }
        }
        for (Map.Entry entry : this.attrMap.entrySet()) {
            MBeanAnalyzer.AttrMethods<Object> attrMethods3 = (MBeanAnalyzer.AttrMethods)entry.getValue();
            if (!mBeanIntrospector.consistent(attrMethods3.getter, attrMethods3.setter)) {
                throw new NotCompliantMBeanException("Getter and setter for " + (String)entry.getKey() + " have inconsistent types");
            }
            continue;
        }
    }
    
    static List<Method> eliminateCovariantMethods(List<Method> list) {
        int size = list.size();
        Method[] a = (Method[])list.toArray(new Method[size]);
        Arrays.sort(a, MBeanAnalyzer.MethodOrder.instance);
        Set<Method> set = Util.newSet();
        for (int i = 1; i < size; ++i) {
            Method method = a[i - 1];
            Method method2 = a[i];
            if (method.getName().equals(method2.getName()) && (Arrays.equals(method.getParameterTypes(), method2.getParameterTypes()) && !set.add(method))) {
                throw new RuntimeException("Internal error: duplicate Method");
            }
        }
        List<Object> list2 = Util.newList(list);
        list2.removeAll(set);
        return list2;
    }
    
    private static class AttrMethods<M>
    {
        M getter;
        M setter;
    }
    
    private static class MethodOrder implements Comparator<Method>
    {
        public static final MBeanAnalyzer.MethodOrder instance;
        
        @Override
        public int compare(Method method, Method method2) {
            int compareTo = method.getName().compareTo(method2.getName());
            if (compareTo != 0) {
                return compareTo;
            }
            Class[] parameterTypes = method.getParameterTypes();
            Class[] parameterTypes2 = method2.getParameterTypes();
            if (parameterTypes.length != parameterTypes2.length) {
                return parameterTypes.length - parameterTypes2.length;
            }
            if (!Arrays.equals(parameterTypes, parameterTypes2)) {
                return Arrays.toString(parameterTypes).compareTo(Arrays.toString(parameterTypes2));
            }
            Class<Object> returnType = method.getReturnType();
            Class<Object> returnType2 = method2.getReturnType();
            if (returnType == returnType2) {
                return 0;
            }
            if (returnType.isAssignableFrom(returnType2)) {
                return -1;
            }
            return 1;
        }
        
        static {
            instance = new MBeanAnalyzer.MethodOrder();
        }
    }
    
    interface MBeanVisitor<M>
    {
        public abstract void visitAttribute(String p0, M p1, M p2);
        
        public abstract void visitOperation(String p0, M p1);
    }
}
